@@ -2,11 +2,8 @@ package Filter::exec ;
 
 use Carp ;
 use Filter::Util::Exec ;
-use strict ;
-use warnings ;
-use vars qw($VERSION) ;
 
-$VERSION = "1.43" ;
+$VERSION = 1.01 ;
 
 sub import
 {
