@@ -1,12 +1,8 @@
 package Filter::tee ;
 
-require 5.002;
 require DynaLoader;
-use strict;
-use warnings;
-use vars qw( @ISA $VERSION);
 @ISA = qw(DynaLoader);
-$VERSION = "1.57" ;
+$VERSION = 1.02 ;
 
 bootstrap Filter::tee ;
 
